@@ -41,19 +41,19 @@ var hasCachedContent = false;
 
 var onMessage = function(msg) {
     switch ( msg.what ) {
-        case 'allFilterListsReloaded':
-            renderFilterLists();
-            break;
+    case 'allFilterListsReloaded':
+        renderFilterLists();
+        break;
 
-        case 'forceUpdateAssetsProgress':
-            renderBusyOverlay(true, msg.progress);
-            if ( msg.done ) {
-                messager.send({ what: 'reloadAllFilters' });
-            }
-            break;
+    case 'forceUpdateAssetsProgress':
+        renderBusyOverlay(true, msg.progress);
+        if ( msg.done ) {
+            messager.send({ what: 'reloadAllFilters' });
+        }
+        break;
 
-        default:
-            break;
+    default:
+        break;
     }
 };
 
@@ -99,7 +99,9 @@ var renderFilterLists = function() {
         }
 
         elem = li.descendants('a:nth-of-type(1)');
-        elem.attr('href', encodeURI(listKey));
+        elem.attr('href', 'asset-viewer.html?url=' + encodeURI(listKey));
+        elem.attr('type', 'text/html');
+        elem.attr('data-listkey', listKey);
         elem.text(listNameFromListKey(listKey) + '\u200E');
 
         elem = li.descendants('a:nth-of-type(2)');
@@ -115,10 +117,20 @@ var renderFilterLists = function() {
             .replace('{{total}}', !isNaN(+entry.entryCount) ? renderNumber(entry.entryCount) : '?');
         elem.text(text);
 
+        // https://github.com/gorhill/uBlock/issues/78
+        // Badge for non-secure connection
+        var remoteURL = listKey;
+        if ( remoteURL.lastIndexOf('http:', 0) !== 0 ) {
+            remoteURL = entry.homeURL || '';
+        }
+        if ( remoteURL.lastIndexOf('http:', 0) === 0 ) {
+            li.descendants('span.status.unsecure').css('display', '');
+        }
+
         // https://github.com/chrisaljoudi/uBlock/issues/104
         var asset = listDetails.cache[listKey] || {};
 
-        // Update status
+        // Badge for update status
         if ( entry.off !== true ) {
             if ( asset.repoObsolete ) {
                 li.descendants('span.status.new').css('display', '');
@@ -334,7 +346,7 @@ var listsContentChanged = function() {
 /******************************************************************************/
 
 var onListCheckboxChanged = function() {
-    var href = uDom(this).parent().descendants('a').first().attr('href');
+    var href = uDom(this).parent().descendants('a').first().attr('data-listkey');
     if ( typeof href !== 'string' ) {
         return;
     }
@@ -347,24 +359,10 @@ var onListCheckboxChanged = function() {
 
 /******************************************************************************/
 
-var onListLinkClicked = function(ev) {
-    messager.send({
-        what: 'gotoURL',
-        details: {
-            url: 'asset-viewer.html?url=' + uDom(this).attr('href'),
-            select: true,
-            index: -1
-        }
-    });
-    ev.preventDefault();
-};
-
-/******************************************************************************/
-
 var onPurgeClicked = function() {
     var button = uDom(this);
     var li = button.parent();
-    var href = li.descendants('a').first().attr('href');
+    var href = li.descendants('a').first().attr('data-listkey');
     if ( !href ) {
         return;
     }
@@ -393,7 +391,7 @@ var selectFilterLists = function(callback) {
     while ( i-- ) {
         li = lis.at(i);
         switches.push({
-            location: li.descendants('a').attr('href'),
+            location: li.descendants('a').attr('data-listkey'),
             off: li.descendants('input').prop('checked') === false
         });
     }
@@ -411,12 +409,8 @@ var buttonApplyHandler = function() {
 
     renderBusyOverlay(true);
 
-    var onReloadDone = function() {
-        messager.send({ what: 'reloadAllFilters' });
-    };
-
     var onSelectionDone = function() {
-        messager.send({ what: 'reloadAllFilters' }, onReloadDone);
+        messager.send({ what: 'reloadAllFilters' });
     };
 
     selectFilterLists(onSelectionDone);
@@ -528,7 +522,6 @@ uDom.onLoad(function() {
     uDom('#buttonUpdate').on('click', buttonUpdateHandler);
     uDom('#buttonPurgeAll').on('click', buttonPurgeAllHandler);
     uDom('#lists').on('change', '.listEntry > input', onListCheckboxChanged);
-    uDom('#lists').on('click', '.listEntry > a:nth-of-type(1)', onListLinkClicked);
     uDom('#lists').on('click', 'span.purge', onPurgeClicked);
     uDom('#externalLists').on('input', externalListsChangeHandler);
     uDom('#externalListsApply').on('click', externalListsApplyHandler);
