@@ -88,7 +88,7 @@ AsyncJobManager.prototype.restartTimer = function() {
     if ( when < this.timerWhen ) {
         clearTimeout(this.timerId);
         this.timerWhen = when;
-        this.timerId = setTimeout(processJobs, Math.max(when - Date.now(), 10));
+        this.timerId = vAPI.setTimeout(processJobs, Math.max(when - Date.now(), 10));
     }
 };
 
@@ -170,28 +170,34 @@ return asyncJobManager;
 /******************************************************************************/
 
 // Update visual of extension icon.
-// A time out is used to coalesce adjacent requests to update badge.
 
-µBlock.updateBadgeAsync = (function(){
-    var µb = µBlock;
+µBlock.updateBadgeAsync = (function() {
+    var tabIdToTimer = Object.create(null);
 
-    // Cache callback definition, it was a bad idea to define this one inside
-    // updateBadgeAsync
     var updateBadge = function(tabId) {
-        var pageStore = µb.pageStoreFromTabId(tabId);
-        if ( pageStore ) {
-            pageStore.updateBadge();
-            return;
+        delete tabIdToTimer[tabId];
+
+        var state = false;
+        var badge = '';
+
+        var pageStore = this.pageStoreFromTabId(tabId);
+        if ( pageStore !== null ) {
+            state = pageStore.getNetFilteringSwitch();
+            if ( state && this.userSettings.showIconBadge && pageStore.perLoadBlockedRequestCount ) {
+                badge = this.utils.formatCount(pageStore.perLoadBlockedRequestCount);
+            }
         }
-        vAPI.setIcon(tabId, 'off', '');
+
+        vAPI.setIcon(tabId, state ? 'on' : 'off', badge);
     };
 
-    var updateBadgeAsync = function(tabId) {
+    return function(tabId) {
+        if ( tabIdToTimer[tabId] ) {
+            return;
+        }
         if ( vAPI.isBehindTheSceneTabId(tabId) ) {
             return;
         }
-        µb.asyncJobs.add('updateBadge-' + tabId, tabId, updateBadge, 250);
+        tabIdToTimer[tabId] = vAPI.setTimeout(updateBadge.bind(this, tabId), 500);
     };
-
-    return updateBadgeAsync;
 })();
