@@ -56,6 +56,7 @@ return {
         advancedUserEnabled: false,
         autoUpdate: true,
         collapseBlocked: true,
+        colorBlindFriendly: false,
         contextMenuEnabled: true,
         dynamicFilteringEnabled: false,
         experimentalEnabled: false,
@@ -71,25 +72,27 @@ return {
     netWhitelist: {},
     netWhitelistModifyTime: 0,
     netWhitelistDefault: [
+        'about-scheme',
         'behind-the-scene',
-        'calculate-linux.org',
-        'calculate-linux.ru',
+	'calculate-linux.org',
+	'calculate-linux.ru',
         'chrome-extension-scheme',
         'chrome-scheme',
+        'loopconversation.about-scheme',
         'opera-scheme',
         ''
     ].join('\n').trim(),
 
     localSettings: {
         blockedRequestCount: 0,
-        allowedRequestCount: 0,
+        allowedRequestCount: 0
     },
     localSettingsModifyTime: 0,
     localSettingsSaveTime: 0,
 
     // read-only
     systemSettings: {
-        compiledMagic: 'cjrthpvolupa',
+        compiledMagic: 'eopszukpnrct',
         selfieMagic: 'spqmeuaftfra'
     },
 
@@ -121,6 +124,10 @@ return {
         'assets/ublock/privacy.txt': {
             title: 'uBlock filters – Privacy',
             group: 'default'
+        },
+        'assets/ublock/unbreak.txt': {
+            title: 'uBlock filters – Unbreak',
+            group: 'default'
         }
     },
 
@@ -131,6 +138,7 @@ return {
     selfieAfter: 23 * oneMinute,
 
     pageStores: {},
+    pageStoresToken: 0,
 
     storageQuota: vAPI.storage.QUOTA_BYTES,
     storageUsed: 0,
@@ -138,13 +146,14 @@ return {
     noopFunc: function(){},
 
     apiErrorCount: 0,
-    contextMenuTarget: '',
     contextMenuClientX: -1,
     contextMenuClientY: -1,
-
     epickerTarget: '',
     epickerEprom: null,
 
+    scriptlets: {
+    },
+
     // so that I don't have to care for last comma
     dummy: 0
 };
