@@ -34,11 +34,16 @@
 // https://github.com/chrisaljoudi/uBlock/issues/464
 if ( document instanceof HTMLDocument === false ) {
     //console.debug('contentscript-end.js > not a HTLMDocument');
-    return false;
+    return;
+}
+
+// I've seen this happens on Firefox
+if ( window.location === null ) {
+    return;
 }
 
 // This can happen
-if ( !vAPI ) {
+if ( typeof vAPI !== 'object' ) {
     //console.debug('contentscript-end.js > vAPI not found');
     return;
 }
@@ -169,7 +174,7 @@ var uBlockCollapser = (function() {
         }
         if ( selectors.length !== 0 ) {
             messager.send({
-                what: 'injectedSelectors',
+                what: 'cosmeticFiltersInjected',
                 type: 'net',
                 hostname: window.location.hostname,
                 selectors: selectors
@@ -201,7 +206,7 @@ var uBlockCollapser = (function() {
             clearTimeout(timer);
             send();
         } else if ( timer === null ) {
-            timer = setTimeout(send, delay || 20);
+            timer = vAPI.setTimeout(send, delay || 20);
         }
     };
 
@@ -227,9 +232,27 @@ var uBlockCollapser = (function() {
         newRequests.push(new BouncingRequest(req.id, tagName, src));
     };
 
-    var addIFrame = function(iframe) {
+    var iframeSourceModified = function(mutations) {
+        var i = mutations.length;
+        while ( i-- ) {
+            addIFrame(mutations[i].target, true);
+        }
+        process();
+    };
+    var iframeSourceObserver = new MutationObserver(iframeSourceModified);
+    var iframeSourceObserverOptions = {
+        attributes: true,
+        attributeFilter: [ 'src' ]
+    };
+
+    var addIFrame = function(iframe, dontObserve) {
+        // https://github.com/gorhill/uBlock/issues/162
+        // Be prepared to deal with possible change of src attribute.
+        if ( dontObserve !== true ) {
+            iframeSourceObserver.observe(iframe, iframeSourceObserverOptions);
+        }
+
         var src = iframe.src;
-        // TODO: niject content script in `about:blank` as well.
         if ( src === '' || typeof src !== 'string' ) {
             return;
         }
@@ -370,7 +393,7 @@ var uBlockCollapser = (function() {
                 processHighHighGenericsAsync();
             }
         }
-        if ( hideSelectors.length ) {
+        if ( hideSelectors.length !== 0 ) {
             addStyleTag(hideSelectors);
         }
         contextNodes.length = 0;
@@ -385,7 +408,7 @@ var uBlockCollapser = (function() {
     // - Injecting a style tag
 
     var addStyleTag = function(selectors) {
-        var selectorStr = selectors.toString();
+        var selectorStr = selectors.join(',\n');
         var style = document.createElement('style');
         // The linefeed before the style block is very important: do no remove!
         style.appendChild(document.createTextNode(selectorStr + '\n{display:none !important;}'));
@@ -396,7 +419,7 @@ var uBlockCollapser = (function() {
         }
         hideElements(selectorStr);
         messager.send({
-            what: 'injectedSelectors',
+            what: 'cosmeticFiltersInjected',
             type: 'cosmetic',
             hostname: window.location.hostname,
             selectors: selectors
@@ -576,7 +599,7 @@ var uBlockCollapser = (function() {
         if ( processHighHighGenericsTimer !== null ) {
             clearTimeout(processHighHighGenericsTimer);
         }
-        processHighHighGenericsTimer = setTimeout(processHighHighGenerics, 300);
+        processHighHighGenericsTimer = vAPI.setTimeout(processHighHighGenerics, 300);
     };
 
     // Extract all ids: these will be passed to the cosmetic filtering
@@ -618,6 +641,11 @@ var uBlockCollapser = (function() {
         var i = nodes.length;
         while ( i-- ) {
             node = nodes[i];
+            // http://jsperf.com/enumerate-classes
+            // Chromium: classList a bit faster than manually enumerating
+            // class names.
+            // Firefox: classList quite slower than manually enumerating
+            // class names.
             vv = node.classList;
             if ( typeof vv !== 'object' ) { continue; }
             j = vv.length || 0;
@@ -687,6 +715,12 @@ var uBlockCollapser = (function() {
             idsFromNodeList(selectNodes('[id]'));
             classesFromNodeList(selectNodes('[class]'));
             retrieveGenericSelectors();
+
+            // This is required: a new element could be matching an already
+            // injected but otherwise inactive cosmetic filter. This means
+            // the already injected cosmetic filter become active (has an
+            // effect on the document), and thus must be logged if needed.
+            messager.send({ what: 'cosmeticFiltersActivated' });
         }
     };
 
@@ -705,7 +739,7 @@ var uBlockCollapser = (function() {
             // I arbitrarily chose 100 ms for now:
             // I have to compromise between the overhead of processing too few
             // nodes too often and the delay of many nodes less often.
-            addedNodeListsTimer = setTimeout(treeMutationObservedHandler, 100);
+            addedNodeListsTimer = vAPI.setTimeout(treeMutationObservedHandler, 100);
         }
     };
 
