@@ -41,7 +41,7 @@ if ( document instanceof HTMLDocument === false ) {
 }
 
 // This can happen
-if ( !vAPI ) {
+if ( typeof vAPI !== 'object' ) {
     //console.debug('contentscript-start.js > vAPI not found');
     return;
 }
@@ -123,6 +123,8 @@ var netFilters = function(details) {
 };
 
 var filteringHandler = function(details) {
+    var styleTagCount = vAPI.styles.length;
+
     vAPI.skipCosmeticFiltering = !details || details.skipCosmeticFiltering;
     if ( details ) {
         if ( details.cosmeticHide.length !== 0 || details.cosmeticDonthide.length !== 0 ) {
@@ -135,6 +137,12 @@ var filteringHandler = function(details) {
         // the browser to flush this script from memory.
     }
 
+    // This is just to inform the background process that cosmetic filters were
+    // actually injected.
+    if ( vAPI.styles.length !== styleTagCount ) {
+        localMessager.send({ what: 'cosmeticFiltersActivated' });
+    }
+
     // https://github.com/chrisaljoudi/uBlock/issues/587
     // If no filters were found, maybe the script was injected before uBlock's
     // process was fully initialized. When this happens, pages won't be
