@@ -1,7 +1,7 @@
 /*******************************************************************************
 
-    µBlock - a browser extension to block requests.
-    Copyright (C) 2014 Raymond Hill
+    uBlock - a browser extension to block requests.
+    Copyright (C) 2014-2015 Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,8 +19,7 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/* jshint bitwise: false */
-/* global vAPI, µBlock */
+/* global µBlock */
 
 /*******************************************************************************
 
@@ -45,178 +44,6 @@ var µb = µBlock;
 /******************************************************************************/
 /******************************************************************************/
 
-var LogEntry = function(details, result) {
-    this.init(details, result);
-};
-
-/******************************************************************************/
-
-var logEntryFactory = function(details, result) {
-    var entry = logEntryJunkyard.pop();
-    if ( entry ) {
-        return entry.init(details, result);
-    }
-    return new LogEntry(details, result);
-};
-
-var logEntryJunkyard = [];
-var logEntryJunkyardMax = 100;
-
-/******************************************************************************/
-
-LogEntry.prototype.init = function(details, result) {
-    this.tstamp = Date.now();
-    this.url = details.requestURL;
-    this.hostname = details.requestHostname;
-    this.type = details.requestType;
-    this.result = result;
-    return this;
-};
-
-/******************************************************************************/
-
-LogEntry.prototype.dispose = function() {
-    this.url = this.hostname = this.type = this.result = '';
-    if ( logEntryJunkyard.length < logEntryJunkyardMax ) {
-        logEntryJunkyard.push(this);
-    }
-};
-
-/******************************************************************************/
-
-var LogBuffer = function() {
-    this.lastReadTime = 0;
-    this.size = 50;
-    this.buffer = null;
-    this.readPtr = 0;
-    this.writePtr = 0;
-};
-
-/******************************************************************************/
-
-var logBufferFactory = function() {
-    return new LogBuffer();
-};
-
-var liveLogBuffers = [];
-
-/******************************************************************************/
-
-LogBuffer.prototype.dispose = function() {
-    if ( this.buffer === null ) {
-        return null;
-    }
-    var entry;
-    var i = this.buffer.length;
-    while ( i-- ) {
-        entry = this.buffer[i];
-        if ( entry instanceof LogEntry ) {
-            entry.dispose();
-        }
-    }
-    this.buffer = null;
-    return null;
-};
-
-/******************************************************************************/
-
-LogBuffer.prototype.start = function() {
-    if ( this.buffer === null ) {
-        this.buffer = new Array(this.size);
-        this.readPtr = 0;
-        this.writePtr = 0;
-        liveLogBuffers.push(this);
-    }
-};
-
-/******************************************************************************/
-
-LogBuffer.prototype.stop = function() {
-    this.dispose();
-    this.buffer = null;
-    // The janitor will remove us from the live pool eventually.
-};
-
-/******************************************************************************/
-
-LogBuffer.prototype.writeOne = function(details, result) {
-    if ( this.buffer === null ) {
-        return;
-    }
-    // Reusing log entry = less memory churning
-    var entry = this.buffer[this.writePtr];
-    if ( entry instanceof LogEntry === false ) {
-        this.buffer[this.writePtr] = logEntryFactory(details, result);
-    } else {
-        entry.init(details, result);
-    }
-    this.writePtr += 1;
-    if ( this.writePtr === this.size ) {
-        this.writePtr = 0;
-    }
-    // Grow the buffer between 1.5x-2x the current size
-    if ( this.writePtr === this.readPtr ) {
-        var toMove = this.buffer.slice(0, this.writePtr);
-        var minSize = Math.ceil(this.size * 1.5);
-        this.size += toMove.length;
-        if ( this.size < minSize ) {
-            this.buffer = this.buffer.concat(toMove, new Array(minSize - this.size));
-            this.writePtr = this.size;
-        } else {
-            this.buffer = this.buffer.concat(toMove);
-            this.writePtr = 0;
-        }
-        this.size = this.buffer.length;
-    }
-};
-
-/******************************************************************************/
-
-LogBuffer.prototype.readAll = function() {
-    var out;
-    if ( this.buffer === null ) {
-        this.start();
-        out = [];
-    } else if ( this.readPtr < this.writePtr ) {
-        out = this.buffer.slice(this.readPtr, this.writePtr);
-    } else if ( this.writePtr < this.readPtr ) {
-        out = this.buffer.slice(this.readPtr).concat(this.buffer.slice(0, this.writePtr));
-    } else {
-        out = [];
-    }
-    this.readPtr = this.writePtr;
-    this.lastReadTime = Date.now();
-    return out;
-};
-
-/******************************************************************************/
-
-var logBufferJanitor = function() {
-    var logBuffer;
-    var obsolete = Date.now() - logBufferObsoleteAfter;
-    var i = liveLogBuffers.length;
-    while ( i-- ) {
-        logBuffer = liveLogBuffers[i];
-        if ( logBuffer.lastReadTime < obsolete ) {
-            logBuffer.stop();
-            liveLogBuffers.splice(i, 1);
-        }
-    }
-    setTimeout(logBufferJanitor, logBufferJanitorPeriod);
-};
-
-// The janitor will look for stale log buffer every 2 minutes.
-var logBufferJanitorPeriod = 2 * 60 * 1000;
-
-// After 30 seconds without being read, a buffer will be considered unused, and
-// thus removed from memory.
-var logBufferObsoleteAfter = 30 * 1000;
-
-setTimeout(logBufferJanitor, logBufferJanitorPeriod);
-
-/******************************************************************************/
-/******************************************************************************/
-
 // To mitigate memory churning
 var netFilteringResultCacheEntryJunkyard = [];
 var netFilteringResultCacheEntryJunkyardMax = 200;
@@ -373,7 +200,7 @@ NetFilteringResultCache.prototype.prune = function() {
 
 NetFilteringResultCache.prototype.pruneAsync = function() {
     if ( this.timer === null ) {
-        this.timer = setTimeout(this.boundPruneAsyncCallback, this.shelfLife * 2);
+        this.timer = vAPI.setTimeout(this.boundPruneAsyncCallback, this.shelfLife * 2);
     }
 };
 
@@ -468,6 +295,8 @@ PageStore.prototype.init = function(tabId) {
     var tabContext = µb.tabContextManager.lookup(tabId);
     this.tabId = tabId;
     this.tabHostname = tabContext.rootHostname;
+    this.title = tabContext.rawURL;
+    this.rawURL = tabContext.rawURL;
     this.hostnameToCountMap = {};
     this.contentLastModified = 0;
     this.frames = {};
@@ -485,12 +314,6 @@ PageStore.prototype.init = function(tabId) {
                                    .matchStringExactType(context, tabContext.normalURL, 'cosmetic-filtering')
                                    .charAt(1) === 'b';
 
-    // Preserve old buffer if there is one already, it may be in use, and
-    // overwritting it would required another read to restart it.
-    if ( this.logBuffer instanceof LogBuffer === false ) {
-        this.logBuffer = logBufferFactory();
-    }
-
     return this;
 };
 
@@ -513,6 +336,7 @@ PageStore.prototype.reuse = function(context) {
     if ( context === 'tabUpdated' ) {
         // As part of https://github.com/chrisaljoudi/uBlock/issues/405
         // URL changed, force a re-evaluation of filtering switch
+        this.rawURL = tabContext.rawURL;
         this.netFilteringReadTime = 0;
         return this;
     }
@@ -533,10 +357,12 @@ PageStore.prototype.dispose = function() {
     // need to release the memory taken by these, which can amount to
     // sizeable enough chunks (especially requests, through the request URL
     // used as a key).
+    this.tabHostname = '';
+    this.title = '';
+    this.rawURL = '';
     this.hostnameToCountMap = null;
     this.disposeFrameStores();
     this.netFilteringCache = this.netFilteringCache.dispose();
-    this.logBuffer = this.logBuffer.dispose();
     if ( pageStoreJunkyard.length < pageStoreJunkyardMax ) {
         pageStoreJunkyard.push(this);
     }
@@ -615,7 +441,7 @@ PageStore.prototype.getNetFilteringSwitch = function() {
     // https://github.com/chrisaljoudi/uBlock/issues/1078
     // Use both the raw and normalized URLs.
     this.netFiltering = µb.getNetFilteringSwitch(tabContext.normalURL);
-    if ( this.netFiltering && tabContext.rawURL !== tabContext.pageURL ) {
+    if ( this.netFiltering && tabContext.rawURL !== tabContext.normalURL ) {
         this.netFiltering = µb.getNetFilteringSwitch(tabContext.rawURL);
     }
     this.netFilteringReadTime = Date.now();
@@ -672,27 +498,24 @@ PageStore.prototype.filterRequest = function(context) {
         return entry.result;
     }
 
-    var result = '';
+    µb.sessionURLFiltering.evaluateZ(context.rootHostname, context.requestURL, context.requestType);
+    var result = µb.sessionURLFiltering.toFilterString();
 
     // Given that:
     // - Dynamic filtering override static filtering
     // - Evaluating dynamic filtering is much faster than static filtering
     // We evaluate dynamic filtering first, and hopefully we can skip
     // evaluation of static filtering.
-    if ( µb.userSettings.advancedUserEnabled ) {
-        var df = µb.sessionFirewall.evaluateCellZY(
-            context.rootHostname,
-            context.requestHostname,
-            context.requestType
-        );
-        if ( df.mustBlockOrAllow() ) {
-            result = df.toFilterString();
+    if ( result === '' && µb.userSettings.advancedUserEnabled ) {
+        µb.sessionFirewall.evaluateCellZY( context.rootHostname, context.requestHostname, context.requestType);
+        if ( µb.sessionFirewall.mustBlockOrAllow() ) {
+            result = µb.sessionFirewall.toFilterString();
         }
     }
 
     // Static filtering never override dynamic filtering
-    if ( result === '' ) {
-        result = µb.staticNetFilteringEngine.matchString(context);
+    if ( result === '' || result.charAt(1) === 'n' ) {
+        result = µb.staticNetFilteringEngine.matchString(context) || result;
     }
 
     //console.debug('cache MISS: PageStore.filterRequest("%s")', context.requestURL);
@@ -705,7 +528,6 @@ PageStore.prototype.filterRequest = function(context) {
     return result;
 };
 
-// Cache only what is worth it if logging is disabled
 // http://jsperf.com/string-indexof-vs-object
 var collapsibleRequestTypes = 'image sub_frame object';
 
@@ -716,27 +538,24 @@ PageStore.prototype.filterRequestNoCache = function(context) {
         return '';
     }
 
-    var result = '';
+    µb.sessionURLFiltering.evaluateZ(context.rootHostname, context.requestURL, context.requestType);
+    var result = µb.sessionURLFiltering.toFilterString();
 
     // Given that:
     // - Dynamic filtering override static filtering
     // - Evaluating dynamic filtering is much faster than static filtering
     // We evaluate dynamic filtering first, and hopefully we can skip
     // evaluation of static filtering.
-    if ( µb.userSettings.advancedUserEnabled ) {
-        var df = µb.sessionFirewall.evaluateCellZY(
-            context.rootHostname,
-            context.requestHostname,
-            context.requestType
-        );
-        if ( df.mustBlockOrAllow() ) {
-            result = df.toFilterString();
+    if ( result === '' && µb.userSettings.advancedUserEnabled ) {
+        µb.sessionFirewall.evaluateCellZY(context.rootHostname, context.requestHostname, context.requestType);
+        if ( µb.sessionFirewall.mustBlockOrAllow() ) {
+            result = µb.sessionFirewall.toFilterString();
         }
     }
 
     // Static filtering never override dynamic filtering
-    if ( result === '' ) {
-        result = µb.staticNetFilteringEngine.matchString(context);
+    if ( result === '' || result.charAt(1) === 'n' ) {
+        result = µb.staticNetFilteringEngine.matchString(context) || result;
     }
 
     return result;
@@ -768,18 +587,6 @@ PageStore.prototype.logRequest = function(context, result) {
         µb.localSettings.blockedRequestCount++;
     }
     µb.localSettingsModifyTime = now;
-    this.logBuffer.writeOne(context, result);
-};
-
-/******************************************************************************/
-
-PageStore.prototype.updateBadge = function() {
-    var netFiltering = this.getNetFilteringSwitch();
-    var badge = '';
-    if ( µb.userSettings.showIconBadge && netFiltering && this.perLoadBlockedRequestCount ) {
-        badge = µb.utils.formatCount(this.perLoadBlockedRequestCount);
-    }
-    vAPI.setIcon(this.tabId, netFiltering ? 'on' : 'off', badge);
 };
 
 // https://www.youtube.com/watch?v=drW8p_dTLD4
