@@ -45,13 +45,21 @@ document.querySelector('#panes > div:nth-of-type(2)').style.setProperty(
     document.querySelector('#panes > div:nth-of-type(1)').offsetHeight + 'px'
 );
 
-// The padlock must be manually positioned:
-// - It's vertical position depends on the height on the title bar.
-document.getElementById('saveRules').style.setProperty(
+// The padlock/eraser must be manually positioned:
+// - Its vertical position depends on the height of the popup title bar
+// - Its horizontal position depends on whether there is a vertical scrollbar.
+document.getElementById('rulesetTools').style.setProperty(
     'top',
-    (document.getElementById('gotoPrefs').getBoundingClientRect().bottom + 4) + 'px'
+    (document.getElementById('gotoPrefs').getBoundingClientRect().bottom + 3) + 'px'
 );
 
+var positionRulesetTools = function() {
+    document.getElementById('rulesetTools').style.setProperty(
+        'left',
+        (document.getElementById('firewallContainer').getBoundingClientRect().left + 3) + 'px'
+    );
+};
+
 // https://github.com/chrisaljoudi/uBlock/issues/996
 // Experimental: mitigate glitchy popup UI: immediately set the firewall pane
 // visibility to its last known state. By default the pane is hidden.
@@ -62,7 +70,7 @@ if ( dfPaneVisibleStored ) {
 
 /******************************************************************************/
 
-var popupData;
+var popupData = {};
 var dfPaneBuilt = false;
 var reIP = /^\d+(?:\.\d+){1,3}$/;
 var reSrcHostnameFromRule = /^d[abn]:([^ ]+) ([^ ]+) ([^ ]+)/;
@@ -291,6 +299,8 @@ var updateAllFirewallCells = function() {
         );
     }
 
+    positionRulesetTools();
+
     uDom('#firewallContainer').toggleClass(
         'dirty',
         popupData.matrixIsDirty === true
@@ -302,7 +312,9 @@ var updateAllFirewallCells = function() {
 var buildAllFirewallRows = function() {
     // Do this before removing the rows
     if ( dfHotspots === null ) {
-        dfHotspots = uDom('#actionSelector').on('click', 'span', setFirewallRuleHandler);
+        dfHotspots = uDom('#actionSelector')
+            .toggleClass('colorBlind', popupData.colorBlindFriendly)
+            .on('click', 'span', setFirewallRuleHandler);
     }
     dfHotspots.detach();
 
@@ -323,12 +335,6 @@ var buildAllFirewallRows = function() {
         dfPaneBuilt = true;
     }
 
-    // The padlock must be manually positioned:
-    // - Its horizontal position depends on whether there is a vertical
-    //   scrollbar.
-    var rect = document.getElementById('firewallContainer').getBoundingClientRect();
-    document.getElementById('saveRules').style.setProperty('left', (rect.left + 4) + 'px');
-
     updateAllFirewallCells();
 };
 
@@ -393,18 +399,16 @@ var renderPopup = function() {
 
     uDom('#appname').text(popupData.appName);
     uDom('#version').text(popupData.appVersion);
-    uDom('body').toggleClass('advancedUser', popupData.advancedUserEnabled);
-
-    uDom('body').toggleClass(
-        'off',
-        (popupData.pageURL === '') ||
-        (!popupData.netFilteringSwitch) ||
-        (popupData.pageHostname === 'behind-the-scene' && !popupData.advancedUserEnabled)
-    );
+    uDom('body')
+        .toggleClass('advancedUser', popupData.advancedUserEnabled)
+        .toggleClass(
+            'off',
+            (popupData.pageURL === '') ||
+            (!popupData.netFilteringSwitch) ||
+            (popupData.pageHostname === 'behind-the-scene' && !popupData.advancedUserEnabled)
+        );
 
     // If you think the `=== true` is pointless, you are mistaken
-    uDom('#gotoLog').toggleClass('enabled', popupData.canRequestLog === true)
-                    .attr('href', 'devtools.html?tabId=' + popupData.tabId);
     uDom('#gotoPick').toggleClass('enabled', popupData.canElementPicker === true);
 
     var text;
@@ -450,7 +454,9 @@ var renderPopup = function() {
     }
 
     uDom('#panes').toggleClass('dfEnabled', dfPaneVisible);
-    uDom('#firewallContainer').toggleClass('minimized', popupData.firewallPaneMinimized);
+    uDom('#firewallContainer')
+        .toggleClass('minimized', popupData.firewallPaneMinimized)
+        .toggleClass('colorBlind', popupData.colorBlindFriendly);
 
     // Build dynamic filtering pane only if in use
     if ( dfPaneVisible ) {
@@ -498,10 +504,9 @@ var toggleNetFilteringSwitch = function(ev) {
 
 var gotoPick = function() {
     messager.send({
-        what: 'gotoPick',
-        tabId: popupData.tabId,
-        select: true
-});
+        what: 'launchElementPicker',
+        tabId: popupData.tabId
+    });
 
     vAPI.closePopup();
 };
@@ -659,6 +664,7 @@ var toggleMinimize = function() {
         name: 'firewallPaneMinimized',
         value: popupData.firewallPaneMinimized
     });
+    positionRulesetTools();
 };
 
 /******************************************************************************/
@@ -674,6 +680,23 @@ var saveFirewallRules = function() {
 
 /******************************************************************************/
 
+var revertFirewallRules = function() {
+    var onFirewallRuleChanged = function(response) {
+        cachePopupData(response);
+        updateAllFirewallCells();
+        hashFromPopupData();
+    };
+    messager.send({
+        what: 'revertFirewallRules',
+        srcHostname: popupData.pageHostname,
+        desHostnames: popupData.hostnameDict,
+        tabId: popupData.tabId
+    }, onFirewallRuleChanged);
+    uDom('#firewallContainer').removeClass('dirty');
+};
+
+/******************************************************************************/
+
 var toggleHostnameSwitch = function() {
     var elem = uDom(this);
     var switchName = elem.attr('id');
@@ -738,7 +761,7 @@ var pollForContentChange = (function() {
         if ( pollTimer !== null ) {
             return;
         }
-        pollTimer = setTimeout(pollCallback, 1500);
+        pollTimer = vAPI.setTimeout(pollCallback, 1500);
     };
 
     return poll;
@@ -795,15 +818,17 @@ var onHideTooltip = function() {
 // Make menu only when popup html is fully loaded
 
 uDom.onLoad(function () {
-    var tabId = null; //If there's no tab ID specified in the query string, it will default to current tab.
+    // If there's no tab id specified in the query string,
+    // it will default to current tab.
+    var tabId = null;
 
     // Extract the tab id of the page this popup is for
     var matches = window.location.search.match(/[\?&]tabId=([^&]+)/);
-    if (matches && matches.length === 2) {
+    if ( matches && matches.length === 2 ) {
         tabId = matches[1];
     }
-
     getPopupData(tabId);
+
     uDom('#switch').on('click', toggleNetFilteringSwitch);
     uDom('#gotoPick').on('click', gotoPick);
     uDom('a[href]').on('click', gotoURL);
@@ -811,6 +836,7 @@ uDom.onLoad(function () {
     uDom('#refresh').on('click', reloadTab);
     uDom('.hnSwitch').on('click', toggleHostnameSwitch);
     uDom('#saveRules').on('click', saveFirewallRules);
+    uDom('#revertRules').on('click', revertFirewallRules);
     uDom('[data-i18n="popupAnyRulePrompt"]').on('click', toggleMinimize);
 
     uDom('body').on('mouseenter', '[data-tip]', onShowTooltip)
