@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    µBlock - a browser extension to block requests.
+    uBlock - a browser extension to block requests.
     Copyright (C) 2014-2015 Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -81,6 +81,16 @@ var onVersionReady = function(lastVersion) {
         );
         µb.saveWhitelist();
     }
+    // https://github.com/gorhill/uBlock/issues/135#issuecomment-96677379
+    // `about:loopconversation` is used by Firefox for its Hello service
+    if ( lastVersion.localeCompare('0.9.5.2') < 0 ) {
+        µb.netWhitelist = µb.whitelistFromString(
+            µb.stringFromWhitelist(µb.netWhitelist) +
+            '\n' +
+            'loopconversation.about-scheme'
+        );
+        µb.saveWhitelist();
+    }
     if ( lastVersion !== vAPI.app.version ) {
         vAPI.storage.set({ version: vAPI.app.version });
     }
@@ -132,11 +142,12 @@ var onUserSettingsReady = function(fetched) {
     // https://github.com/chrisaljoudi/uBlock/issues/540
     // Disabling local mirroring for the time being
     userSettings.experimentalEnabled = false;
-    µb.mirrors.toggle(false /* userSettings.experimentalEnabled */);
 
     µb.contextMenu.toggle(userSettings.contextMenuEnabled);
     µb.permanentFirewall.fromString(fetched.dynamicFilteringString);
     µb.sessionFirewall.assign(µb.permanentFirewall);
+    µb.permanentURLFiltering.fromString(fetched.urlFilteringString);
+    µb.sessionURLFiltering.assign(µb.permanentURLFiltering);
     µb.hnSwitches.fromString(fetched.hostnameSwitchesString);
 
     // Remove obsolete setting
@@ -216,6 +227,7 @@ return function() {
     var fetchableProps = {
         'compiledMagic': '',
         'dynamicFilteringString': '',
+        'urlFilteringString': '',
         'hostnameSwitchesString': '',
         'lastRestoreFile': '',
         'lastRestoreTime': 0,
