@@ -572,6 +572,47 @@ FilterPlainHnAnchored.fromSelfie = function(s) {
 
 /******************************************************************************/
 
+// https://github.com/gorhill/uBlock/issues/142
+
+var FilterPlainHnAnchoredHostname = function(s, hostname) {
+    this.s = s;
+    this.hostname = hostname;
+};
+
+FilterPlainHnAnchoredHostname.prototype.match = function(url, tokenBeg) {
+    if ( pageHostnameRegister.slice(-this.hostname.length) !== this.hostname ) {
+        return false;
+    }
+    if ( url.substr(tokenBeg, this.s.length) !== this.s ) {
+        return false;
+    }
+    // Valid only if hostname-valid characters to the left of token
+    var pos = url.indexOf('://');
+    return pos !== -1 &&
+           reURLPostHostnameAnchors.test(url.slice(pos + 3, tokenBeg)) === false;
+};
+
+FilterPlainHnAnchoredHostname.fid = FilterPlainHnAnchoredHostname.prototype.fid = '||ah';
+
+FilterPlainHnAnchoredHostname.prototype.toString = function() {
+    return '||' + this.s;
+};
+
+FilterPlainHnAnchoredHostname.prototype.toSelfie = function() {
+    return this.s + '\t' + this.hostname;
+};
+
+FilterPlainHnAnchoredHostname.compile = function(details, hostname) {
+    return details.f + '\t' + hostname;
+};
+
+FilterPlainHnAnchoredHostname.fromSelfie = function(s) {
+    var pos = s.indexOf('\t');
+    return new FilterPlainHnAnchoredHostname(s.slice(0, pos), s.slice(pos + 1));
+};
+
+/******************************************************************************/
+
 // Generic filter
 
 var FilterGeneric = function(s, anchor) {
@@ -833,13 +874,21 @@ FilterHostnameDict.prototype.meltBucket = function(len, bucket) {
     } else {
         var offset = 0;
         while ( offset < bucket.length ) {
-            map[bucket.substring(offset, len)] = true;
+            map[bucket.substr(offset, len)] = true;
             offset += len;
         }
     }
     return map;
 };
 
+FilterHostnameDict.prototype.freezeBucket = function(bucket) {
+    var hostnames = Object.keys(bucket);
+    if ( hostnames[0].length * hostnames.length < this.cutoff ) {
+        return ' ' + hostnames.join(' ') + ' ';
+    }
+    return hostnames.sort().join('');
+};
+
 // How the key is derived dictates the number and size of buckets:
 // - more bits = more buckets = higher memory footprint
 // - less bits = less buckets = lower memory footprint
@@ -887,7 +936,7 @@ FilterHostnameDict.prototype.add = function(hn) {
         return true;
     }
     if ( typeof bucket === 'string' ) {
-        bucket = this.dict[key] = this.meltBucket(hn.len, bucket);
+        bucket = this.dict[key] = this.meltBucket(hn.length, bucket);
     }
     if ( bucket.hasOwnProperty(hn) ) {
         return false;
@@ -899,18 +948,11 @@ FilterHostnameDict.prototype.add = function(hn) {
 
 FilterHostnameDict.prototype.freeze = function() {
     var buckets = this.dict;
-    var bucket, hostnames, len;
+    var bucket;
     for ( var key in buckets ) {
         bucket = buckets[key];
-        if ( typeof bucket !== 'object' ) {
-            continue;
-        }
-        hostnames = Object.keys(bucket);
-        len = hostnames[0].length * hostnames.length;
-        if ( hostnames[0].length * hostnames.length < this.cutoff ) {
-            buckets[key] = ' ' + hostnames.join(' ') + ' ';
-        } else {
-            buckets[key] = hostnames.sort().join('');
+        if ( typeof bucket === 'object' ) {
+            buckets[key] = this.freezeBucket(bucket);
         }
     }
 };
@@ -924,7 +966,7 @@ FilterHostnameDict.prototype.matchesExactly = function(hn) {
         return false;
     }
     if ( typeof bucket === 'object' ) {
-        return bucket.hasOwnProperty(hn);
+        bucket = this.dict[key] = this.freezeBucket(bucket);
     }
     if ( bucket.charAt(0) === ' ' ) {
         return bucket.indexOf(' ' + hn + ' ') !== -1;
@@ -1142,6 +1184,9 @@ var getHostnameBasedFilterClass = function(details) {
     if ( details.anchor > 0 ) {
         return FilterPlainRightAnchoredHostname;
     }
+    if ( details.hostnameAnchored ) {
+        return FilterPlainHnAnchoredHostname;
+    }
     if ( details.tokenBeg === 0 ) {
         return FilterPlainPrefix0Hostname;
     }
@@ -1289,7 +1334,7 @@ FilterParser.prototype.parseOptions = function(s) {
             continue;
         }
         if ( opt === 'elemhide' ) {
-            if ( this.action !== AllowAction ) {
+            if ( this.action === AllowAction ) {
                 this.parseOptType('elemhide', false);
                 this.action = BlockAction;
                 continue;
@@ -1573,25 +1618,26 @@ FilterContainer.prototype.freeze = function() {
 /******************************************************************************/
 
 FilterContainer.prototype.factories = {
-     '[]': FilterBucket,
-      'a': FilterPlain,
-     'ah': FilterPlainHostname,
-     '0a': FilterPlainPrefix0,
-    '0ah': FilterPlainPrefix0Hostname,
-     '1a': FilterPlainPrefix1,
-    '1ah': FilterPlainPrefix1Hostname,
-     '|a': FilterPlainLeftAnchored,
-    '|ah': FilterPlainLeftAnchoredHostname,
-     'a|': FilterPlainRightAnchored,
-    'a|h': FilterPlainRightAnchoredHostname,
-    '||a': FilterPlainHnAnchored,
-     '//': FilterRegex,
-    '//h': FilterRegexHostname,
-    '{h}': FilterHostnameDict,
-      '_': FilterGeneric,
-     '_h': FilterGenericHostname,
-    '||_': FilterGenericHnAnchored,
-   '||_h': FilterGenericHnAnchoredHostname
+      '[]': FilterBucket,
+       'a': FilterPlain,
+      'ah': FilterPlainHostname,
+      '0a': FilterPlainPrefix0,
+     '0ah': FilterPlainPrefix0Hostname,
+      '1a': FilterPlainPrefix1,
+     '1ah': FilterPlainPrefix1Hostname,
+      '|a': FilterPlainLeftAnchored,
+     '|ah': FilterPlainLeftAnchoredHostname,
+      'a|': FilterPlainRightAnchored,
+     'a|h': FilterPlainRightAnchoredHostname,
+     '||a': FilterPlainHnAnchored,
+    '||ah': FilterPlainHnAnchoredHostname,
+      '//': FilterRegex,
+     '//h': FilterRegexHostname,
+     '{h}': FilterHostnameDict,
+       '_': FilterGeneric,
+      '_h': FilterGenericHostname,
+     '||_': FilterGenericHnAnchored,
+    '||_h': FilterGenericHnAnchoredHostname
 };
 
 /******************************************************************************/
