@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    µBlock - a browser extension to block requests.
+    uBlock - a browser extension to block requests.
     Copyright (C) 2014-2015 Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -83,6 +83,12 @@
 
 /******************************************************************************/
 
+µBlock.savePermanentURLFilteringRules = function() {
+    this.keyvalSetOne('urlFilteringString', this.permanentURLFiltering.toString());
+};
+
+/******************************************************************************/
+
 µBlock.saveHostnameSwitches = function() {
     this.keyvalSetOne('hostnameSwitchesString', this.hnSwitches.toString());
 };
@@ -102,19 +108,40 @@
 // This will remove all unused filter list entries from
 // µBlock.remoteBlacklists`. This helps reduce the size of backup files.
 
-µBlock.extractSelectedFilterLists = function() {
-    var r = JSON.parse(JSON.stringify(this.remoteBlacklists));
+µBlock.extractSelectedFilterLists = function(callback) {
+    var µb = this;
 
-    for ( var path in r ) {
-        if ( r.hasOwnProperty(path) === false ) {
-            continue;
+    var onBuiltinListsLoaded = function(details) {
+        var builtin;
+        try {
+            builtin = JSON.parse(details.content);
+        } catch (e) {
+            builtin = {};
         }
-        if ( r[path].off !== false ) {
-            delete r[path];
-        }
-    }
 
-    return r;
+        var result = JSON.parse(JSON.stringify(µb.remoteBlacklists));
+        var builtinPath;
+        var defaultState;
+
+        for ( var path in result ) {
+            if ( result.hasOwnProperty(path) === false ) {
+                continue;
+            }
+            builtinPath = path.replace(/^assets\/thirdparties\//, '');
+            defaultState = builtin.hasOwnProperty(builtinPath) === false ||
+                           builtin[builtinPath].off === true;
+            if ( result[path].off === true && result[path].off === defaultState ) {
+                delete result[path];
+            }
+        }
+
+        callback(result);
+    };
+
+    // https://github.com/gorhill/uBlock/issues/63
+    // Get built-in block lists: this will help us determine whether a
+    // specific list must be included in the result.
+    this.assets.get('assets/ublock/filter-lists.json', onBuiltinListsLoaded);
 };
 
 /******************************************************************************/
@@ -131,33 +158,30 @@
 
 /******************************************************************************/
 
-µBlock.appendUserFilters = function(filter) {
-    if ( filter.length === 0 ) {
+µBlock.appendUserFilters = function(filters) {
+    if ( filters.length === 0 ) {
         return;
     }
 
     var µb = this;
 
-    var onCompiledListLoaded = function(details) {
+    var onSaved = function() {
+        var compiledFilters = µb.compileFilters(filters);
         var snfe = µb.staticNetFilteringEngine;
         var cfe = µb.cosmeticFilteringEngine;
         var acceptedCount = snfe.acceptedCount + cfe.acceptedCount;
         var duplicateCount = snfe.duplicateCount + cfe.duplicateCount;
-        µb.applyCompiledFilters(details.content);
+        µb.applyCompiledFilters(compiledFilters);
         var entry = µb.remoteBlacklists[µb.userFiltersPath];
-        entry.entryCount = snfe.acceptedCount + cfe.acceptedCount - acceptedCount;
-        entry.entryUsedCount = entry.entryCount - snfe.duplicateCount - cfe.duplicateCount + duplicateCount;
+        var deltaEntryCount = snfe.acceptedCount + cfe.acceptedCount - acceptedCount;
+        var deltaEntryUsedCount = deltaEntryCount - (snfe.duplicateCount + cfe.duplicateCount - duplicateCount);
+        entry.entryCount += deltaEntryCount;
+        entry.entryUsedCount += deltaEntryUsedCount;
+        vAPI.storage.set({ 'remoteBlacklists': µb.remoteBlacklists });
         µb.staticNetFilteringEngine.freeze();
         µb.cosmeticFilteringEngine.freeze();
     };
 
-    var onSaved = function(details) {
-        if ( details.error ) {
-            return;
-        }
-        µb.getCompiledFilterList(µb.userFiltersPath, onCompiledListLoaded);
-    };
-
     var onLoaded = function(details) {
         if ( details.error ) {
             return;
@@ -166,7 +190,7 @@
         // If we reached this point, the filter quite probably needs to be
         // added for sure: do not try to be too smart, trying to avoid
         // duplicates at this point may lead to more issues.
-        µb.saveUserFilters(details.content.trim() + '\n\n' + filter.trim(), onSaved);
+        µb.saveUserFilters(details.content.trim() + '\n\n' + filters.trim(), onSaved);
     };
 
     this.loadUserFilters(onLoaded);
@@ -193,20 +217,25 @@
         var lists = store.remoteBlacklists;
         var locations = Object.keys(lists);
         var location, availableEntry, storedEntry;
+        var off;
 
         while ( location = locations.pop() ) {
             storedEntry = lists[location];
+            off = storedEntry.off === true;
             // New location?
             if ( relocationMap.hasOwnProperty(location) ) {
                 µb.purgeFilterList(location);
                 location = relocationMap[location];
+                if ( off && lists.hasOwnProperty(location) ) {
+                    off = lists[location].off === true;
+                }
             }
             availableEntry = availableLists[location];
             if ( availableEntry === undefined ) {
                 µb.purgeFilterList(location);
                 continue;
             }
-            availableEntry.off = storedEntry.off || false;
+            availableEntry.off = off;
             µb.assets.setHomeURL(location, availableEntry.homeURL);
             if ( storedEntry.entryCount !== undefined ) {
                 availableEntry.entryCount = storedEntry.entryCount;
