@@ -55,13 +55,15 @@ var µb = µBlock;
         return uri.normalizedURI();
     }
 
-    var url = 'http://' + scheme + '-scheme/';
+    var fakeHostname = scheme + '-scheme';
 
     if ( uri.hostname !== '' ) {
-        url += uri.hostname + '/';
+        fakeHostname = uri.hostname + '.' + fakeHostname;
+    } else if ( scheme === 'about' && uri.path !== '' ) {
+        fakeHostname = uri.path + '.' + fakeHostname;
     }
 
-    return url;
+    return 'http://' + fakeHostname + '/';
 };
 
 /******************************************************************************/
@@ -136,6 +138,14 @@ housekeep itself.
 
     var gcPeriod = 10 * 60 * 1000;
 
+    // A pushed entry is removed from the stack unless it is committed with
+    // a set time.
+    var StackEntry = function(url, commit) {
+        this.url = url;
+        this.committed = commit;
+        this.tstamp = Date.now();
+    };
+
     var TabContext = function(tabId) {
         this.tabId = tabId.toString();
         this.stack = [];
@@ -143,9 +153,8 @@ housekeep itself.
         this.normalURL =
         this.rootHostname =
         this.rootDomain = '';
-        this.timer = null;
-        this.onTabCallback = null;
-        this.onTimerCallback = null;
+        this.commitTimer = null;
+        this.gcTimer = null;
 
         tabContexts[tabId] = this;
     };
@@ -154,27 +163,49 @@ housekeep itself.
         if ( vAPI.isBehindTheSceneTabId(this.tabId) ) {
             return;
         }
-        if ( this.timer !== null ) {
-            clearTimeout(this.timer);
-            this.timer = null;
+        if ( this.gcTimer !== null ) {
+            clearTimeout(this.gcTimer);
+            this.gcTimer = null;
         }
         delete tabContexts[this.tabId];
     };
 
     TabContext.prototype.onTab = function(tab) {
         if ( tab ) {
-            this.timer = setTimeout(this.onTimerCallback, gcPeriod);
+            this.gcTimer = vAPI.setTimeout(this.onGC.bind(this), gcPeriod);
         } else {
             this.destroy();
         }
     };
 
-    TabContext.prototype.onTimer = function() {
-        this.timer = null;
+    TabContext.prototype.onGC = function() {
+        this.gcTimer = null;
         if ( vAPI.isBehindTheSceneTabId(this.tabId) ) {
             return;
         }
-        vAPI.tabs.get(this.tabId, this.onTabCallback);
+        vAPI.tabs.get(this.tabId, this.onTab.bind(this));
+    };
+
+    // https://github.com/gorhill/uBlock/issues/248
+    // Stack entries have to be committed to stick. Non-committed stack
+    // entries are removed after a set delay.
+    TabContext.prototype.onCommit = function() {
+        if ( vAPI.isBehindTheSceneTabId(this.tabId) ) {
+            return;
+        }
+        this.commitTimer = null;
+        // Remove uncommitted entries at the top of the stack.
+        var i = this.stack.length;
+        while ( i-- ) {
+            if ( this.stack[i].committed ) {
+                break;
+            }
+        }
+        i += 1;
+        if ( i < this.stack.length ) {
+            this.stack.length = i;
+            this.update();
+        }
     };
 
     // This takes care of orphanized tab contexts. Can't be started for all
@@ -184,22 +215,21 @@ housekeep itself.
         if ( vAPI.isBehindTheSceneTabId(this.tabId) ) {
             return;
         }
-        this.onTabCallback = this.onTab.bind(this);
-        this.onTimerCallback = this.onTimer.bind(this);
-        this.timer = setTimeout(this.onTimerCallback, gcPeriod);
+        this.gcTimer = vAPI.setTimeout(this.onGC.bind(this), gcPeriod);
     };
 
-    // Update just force all properties to be updated to match the most current
+    // Update just force all properties to be updated to match the most recent
     // root URL.
     TabContext.prototype.update = function() {
         if ( this.stack.length === 0 ) {
             this.rawURL = this.normalURL = this.rootHostname = this.rootDomain = '';
-        } else {
-            this.rawURL = this.stack[this.stack.length - 1];
-            this.normalURL = µb.normalizePageURL(this.tabId, this.rawURL);
-            this.rootHostname = µb.URI.hostnameFromURI(this.normalURL);
-            this.rootDomain = µb.URI.domainFromHostname(this.rootHostname);
+            return;
         }
+        var stackEntry = this.stack[this.stack.length - 1];
+        this.rawURL = stackEntry.url;
+        this.normalURL = µb.normalizePageURL(this.tabId, this.rawURL);
+        this.rootHostname = µb.URI.hostnameFromURI(this.normalURL);
+        this.rootDomain = µb.URI.domainFromHostname(this.rootHostname);
     };
 
     // Called whenever a candidate root URL is spotted for the tab.
@@ -207,8 +237,16 @@ housekeep itself.
         if ( vAPI.isBehindTheSceneTabId(this.tabId) ) {
             return;
         }
-        this.stack.push(url);
+        var count = this.stack.length;
+        if ( count !== 0 && this.stack[count - 1].url === url ) {
+            return;
+        }
+        this.stack.push(new StackEntry(url));
         this.update();
+        if ( this.commitTimer === null ) {
+            clearTimeout(this.commitTimer);
+        }
+        this.commitTimer = vAPI.setTimeout(this.onCommit.bind(this), 1000);
     };
 
     // Called when a former push is a false positive:
@@ -219,22 +257,20 @@ housekeep itself.
         }
         // We are not going to unpush if there is no other candidate, the
         // point of unpush is to make space for a better candidate.
-        if ( this.stack.length === 1 ) {
+        var i = this.stack.length;
+        if ( i === 1 ) {
             return;
         }
-        var pos = this.stack.indexOf(url);
-        if ( pos === -1 ) {
+        while ( i-- ) {
+            if ( this.stack[i].url !== url ) {
+                continue;
+            }
+            this.stack.splice(i, 1);
+            if ( i === this.stack.length ) {
+                this.update();
+            }
             return;
         }
-        this.stack.splice(pos, 1);
-        if ( this.stack.length === 0 ) {
-            this.destroy();
-            return;
-        }
-        if ( pos !== this.stack.length ) {
-            return;
-        }
-        this.update();
     };
 
     // This tells that the url is definitely the one to be associated with the
@@ -244,7 +280,7 @@ housekeep itself.
         if ( vAPI.isBehindTheSceneTabId(this.tabId) ) {
             return;
         }
-        this.stack = [url];
+        this.stack = [new StackEntry(url, true)];
         this.update();
     };
 
@@ -322,7 +358,7 @@ housekeep itself.
     // Behind-the-scene tab context
     (function() {
         var entry = new TabContext(vAPI.noTabId);
-        entry.stack.push('');
+        entry.stack.push(new StackEntry('', true));
         entry.rawURL = '';
         entry.normalURL = µb.normalizePageURL(entry.tabId);
         entry.rootHostname = µb.URI.hostnameFromURI(entry.normalURL);
@@ -452,7 +488,7 @@ vAPI.tabs.onPopup = function(details) {
 
     // Check user switch first
     if ( µb.hnSwitches.evaluateZ('no-popups', openerHostname) ) {
-        result = 'ub:no-popups true';
+        result = 'ub:no-popups: ' + µb.hnSwitches.z + ' true';
     }
 
     // https://github.com/chrisaljoudi/uBlock/issues/323
@@ -471,6 +507,14 @@ vAPI.tabs.onPopup = function(details) {
     if ( pageStore ) {
         pageStore.logRequest(context, result);
     }
+    µb.logger.writeOne(
+        details.openerTabId,
+        'net',
+        result,
+        'popup',
+        targetURL,
+        openerHostname
+    );
 
     // Not blocked
     if ( µb.isAllowResult(result) ) {
@@ -478,6 +522,9 @@ vAPI.tabs.onPopup = function(details) {
     }
 
     // Blocked
+    if ( µb.userSettings.showIconBadge ) {
+        µb.updateBadgeAsync(details.openerTabId);
+    }
 
     // It is a popup, block and remove the tab.
     µb.unbindTabFromPageStats(details.targetTabId);
@@ -494,9 +541,7 @@ vAPI.tabs.registerListeners();
 // Create an entry for the tab if it doesn't exist.
 
 µb.bindTabToPageStats = function(tabId, context) {
-    if ( vAPI.isBehindTheSceneTabId(tabId) === false ) {
-        this.updateBadgeAsync(tabId);
-    }
+    this.updateBadgeAsync(tabId);
 
     // Do not create a page store for URLs which are of no interests
     if ( µb.tabContextManager.exists(tabId) === false ) {
@@ -509,6 +554,8 @@ vAPI.tabs.registerListeners();
 
     // Tab is not bound
     if ( !pageStore ) {
+        this.updateTitle(tabId);
+        this.pageStoresToken = Date.now();
         return this.pageStores[tabId] = this.PageStore.factory(tabId);
     }
 
@@ -529,6 +576,9 @@ vAPI.tabs.registerListeners();
     // be all reset.
     pageStore.reuse(context);
 
+    this.updateTitle(tabId);
+    this.pageStoresToken = Date.now();
+
     return pageStore;
 };
 
@@ -540,13 +590,14 @@ vAPI.tabs.registerListeners();
     if ( pageStore !== undefined ) {
         pageStore.dispose();
         delete this.pageStores[tabId];
+        this.pageStoresToken = Date.now();
     }
 };
 
 /******************************************************************************/
 
 µb.pageStoreFromTabId = function(tabId) {
-    return this.pageStores[tabId];
+    return this.pageStores[tabId] || null;
 };
 
 /******************************************************************************/
@@ -554,8 +605,77 @@ vAPI.tabs.registerListeners();
 // Permanent page store for behind-the-scene requests. Must never be removed.
 
 µb.pageStores[vAPI.noTabId] = µb.PageStore.factory(vAPI.noTabId);
+µb.pageStores[vAPI.noTabId].title = vAPI.i18n('logBehindTheScene');
 
 /******************************************************************************/
+
+µb.updateTitle = (function() {
+    var tabIdToTimer = Object.create(null);
+    var tabIdToTryCount = Object.create(null);
+    var delay = 499;
+
+    var tryNoMore = function(tabId) {
+        delete tabIdToTryCount[tabId];
+    };
+
+    var tryAgain = function(tabId) {
+        var count = tabIdToTryCount[tabId];
+        if ( count === undefined ) {
+            return false;
+        }
+        if ( count === 1 ) {
+            delete tabIdToTryCount[tabId];
+            return false;
+        }
+        tabIdToTryCount[tabId] = count - 1;
+        tabIdToTimer[tabId] = vAPI.setTimeout(updateTitle.bind(µb, tabId), delay);
+        return true;
+    };
+
+    var onTabReady = function(tabId, tab) {
+        if ( !tab ) {
+            return tryNoMore(tabId);
+        }
+        var pageStore = this.pageStoreFromTabId(tabId);
+        if ( pageStore === null ) {
+            return tryNoMore(tabId);
+        }
+        // Firefox needs this: if you detach a tab, the new tab won't have
+        // its rawURL set. Concretely, this causes the logger to report an
+        // entry to itself in the logger's tab selector.
+        // TODO: Investigate for a fix vAPI-side.
+        pageStore.rawURL = tab.url;
+        this.pageStoresToken = Date.now();
+        if ( !tab.title && tryAgain(tabId) ) {
+            return;
+        }
+        // https://github.com/gorhill/uMatrix/issues/225
+        // Sometimes title changes while page is loading.
+        var settled = tab.title && tab.title === pageStore.title;
+        pageStore.title = tab.title || tab.url || '';
+        this.pageStoresToken = Date.now();
+        if ( settled || !tryAgain(tabId) ) {
+            tryNoMore(tabId);
+        }
+    };
+
+    var updateTitle = function(tabId) {
+        delete tabIdToTimer[tabId];
+        vAPI.tabs.get(tabId, onTabReady.bind(this, tabId));
+    };
+
+    return function(tabId) {
+        if ( vAPI.isBehindTheSceneTabId(tabId) ) {
+            return;
+        }
+        if ( tabIdToTimer[tabId] ) {
+            clearTimeout(tabIdToTimer[tabId]);
+        }
+        tabIdToTimer[tabId] = vAPI.setTimeout(updateTitle.bind(this, tabId), delay);
+        tabIdToTryCount[tabId] = 5;
+    };
+})();
+
 /******************************************************************************/
 
 // Stale page store entries janitor
@@ -591,12 +711,11 @@ var pageStoreJanitor = function() {
     }
     pageStoreJanitorSampleAt = n;
 
-    setTimeout(pageStoreJanitor, pageStoreJanitorPeriod);
+    vAPI.setTimeout(pageStoreJanitor, pageStoreJanitorPeriod);
 };
 
-setTimeout(pageStoreJanitor, pageStoreJanitorPeriod);
+vAPI.setTimeout(pageStoreJanitor, pageStoreJanitorPeriod);
 
-/******************************************************************************/
 /******************************************************************************/
 
 })();
