@@ -92,6 +92,7 @@ var onBeforeRequest = function(details) {
     // Possible outcomes: blocked, allowed-passthru, allowed-mirror
 
     pageStore.logRequest(requestContext, result);
+    µb.logger.writeOne(tabId, 'net', result, requestType, requestURL, requestContext.rootHostname);
 
     // Not blocked
     if ( µb.isAllowResult(result) ) {
@@ -160,7 +161,7 @@ var onBeforeRootFrameRequest = function(details) {
 
     // Permanently unrestricted?
     if ( result === '' && µb.hnSwitches.evaluateZ('no-strict-blocking', requestHostname) ) {
-        result = 'ua:no-strict-blocking on';
+        result = 'ua:no-strict-blocking: ' + µb.hnSwitches.z + ' true';
     }
 
     // Temporarily whitelisted?
@@ -186,6 +187,7 @@ var onBeforeRootFrameRequest = function(details) {
     if ( pageStore ) {
         pageStore.logRequest(context, result);
     }
+    µb.logger.writeOne(tabId, 'net', result, 'main_frame', requestURL, context.rootHostname);
 
     // Not blocked
     if ( µb.isAllowResult(result) ) {
@@ -277,6 +279,7 @@ var onBeforeBehindTheSceneRequest = function(details) {
     }
 
     pageStore.logRequest(context, result);
+    µb.logger.writeOne(vAPI.noTabId, 'net', result, details.type, details.url, context.rootHostname);
 
     // Not blocked
     if ( µb.isAllowResult(result) ) {
@@ -315,16 +318,17 @@ var onHeadersReceived = function(details) {
         return;
     }
 
-    // Frame id of frame request is the their own id, while the request is made
+    // Frame id of frame request is their own id, while the request is made
     // in the context of the parent.
     var context = pageStore.createContextFromFrameId(details.parentFrameId);
-    context.requestURL = details.url + '{inline-script}';
+    context.requestURL = details.url;
     context.requestHostname = details.hostname;
     context.requestType = 'inline-script';
 
     var result = pageStore.filterRequestNoCache(context);
 
     pageStore.logRequest(context, result);
+    µb.logger.writeOne(tabId, 'net', result, 'inline-script', details.url, context.rootHostname);
 
     // Don't block
     if ( µb.isAllowResult(result) ) {
@@ -345,15 +349,9 @@ var onHeadersReceived = function(details) {
 
 var onRootFrameHeadersReceived = function(details) {
     var tabId = details.tabId;
-    var requestURL = details.url;
-    var requestHostname = details.hostname;
     var µb = µBlock;
 
-    // Check if the main_frame is a download
-    // ...
-    if ( headerValue(details.responseHeaders, 'content-disposition').lastIndexOf('attachment', 0) === 0 ) {
-        µb.tabContextManager.unpush(tabId, requestURL);
-    }
+    µb.tabContextManager.push(tabId, details.url);
 
     // Lookup the page store associated with this tab id.
     var pageStore = µb.pageStoreFromTabId(tabId);
@@ -363,13 +361,14 @@ var onRootFrameHeadersReceived = function(details) {
     // I can't think of how pageStore could be null at this point.
 
     var context = pageStore.createContextFromPage();
-    context.requestURL = requestURL + '{inline-script}';
-    context.requestHostname = requestHostname;
+    context.requestURL = details.url;
+    context.requestHostname = details.hostname;
     context.requestType = 'inline-script';
 
     var result = pageStore.filterRequestNoCache(context);
 
     pageStore.logRequest(context, result);
+    µb.logger.writeOne(tabId, 'net', result, 'inline-script', details.url, context.rootHostname);
 
     // Don't block
     if ( µb.isAllowResult(result) ) {
@@ -388,33 +387,11 @@ var onRootFrameHeadersReceived = function(details) {
 
 /******************************************************************************/
 
-var headerValue = function(headers, name) {
-    var i = headers.length;
-    while ( i-- ) {
-        if ( headers[i].name.toLowerCase() === name ) {
-            return headers[i].value.trim();
-        }
-    }
-    return '';
-};
-
-/******************************************************************************/
-
 vAPI.net.onBeforeRequest = {
     urls: [
         'http://*/*',
         'https://*/*'
     ],
-    types: [
-        "main_frame",
-        "sub_frame",
-        'stylesheet',
-        "script",
-        "image",
-        "object",
-        "xmlhttprequest",
-        "other"
-    ],
     extra: [ 'blocking' ],
     callback: onBeforeRequest
 };
