@@ -180,6 +180,12 @@ var matchWhitelistDirective = function(url, hostname, directive) {
     var line, matches, key, directive;
     for ( var i = 0; i < lines.length; i++ ) {
         line = lines[i].trim();
+        // https://github.com/gorhill/uBlock/issues/171
+        // Skip empty lines
+        if ( line === '' ) {
+            continue;
+        }
+
         // Don't throw out commented out lines: user might want to fix them
         if ( line.charAt(0) === '#' ) {
             key = '#';
@@ -208,9 +214,14 @@ var matchWhitelistDirective = function(url, hostname, directive) {
             }
         }
 
+        // https://github.com/gorhill/uBlock/issues/171
+        // Skip empty keys
+        if ( key === '' ) {
+            continue;
+        }
+
         // Be sure this stays fixed:
         // https://github.com/chrisaljoudi/uBlock/issues/185
-
         if ( whitelist.hasOwnProperty(key) === false ) {
             whitelist[key] = [];
         }
@@ -243,8 +254,8 @@ var matchWhitelistDirective = function(url, hostname, directive) {
 
     // Pre-change
     switch ( name ) {
-        default:
-            break;
+    default:
+        break;
     }
 
     // Change
@@ -252,23 +263,18 @@ var matchWhitelistDirective = function(url, hostname, directive) {
 
     // Post-change
     switch ( name ) {
-        case 'collapseBlocked':
-            if ( value === false ) {
-                this.cosmeticFilteringEngine.removeFromSelectorCache('*', 'net');
-            }
-            break;
-        case 'contextMenuEnabled':
-            this.contextMenu.toggle(value);
-            break;
-        case 'experimentalEnabled':
-            if ( typeof this.mirrors === 'object' ) {
-                // https://github.com/chrisaljoudi/uBlock/issues/540
-                // Disabling local mirroring for the time being
-                this.mirrors.toggle(false /* value */);
-            }
-            break;
-        default:
-            break;
+    case 'collapseBlocked':
+        if ( value === false ) {
+            this.cosmeticFilteringEngine.removeFromSelectorCache('*', 'net');
+        }
+        break;
+    case 'contextMenuEnabled':
+        this.contextMenu.toggle(value);
+        break;
+    case 'experimentalEnabled':
+        break;
+    default:
+        break;
     }
 
     this.saveUserSettings();
@@ -277,8 +283,14 @@ var matchWhitelistDirective = function(url, hostname, directive) {
 /******************************************************************************/
 
 µBlock.elementPickerExec = function(tabId, targetElement) {
+    if ( vAPI.isBehindTheSceneTabId(tabId) ) {
+        return;
+    }
     this.epickerTarget = targetElement || '';
-    vAPI.tabs.injectScript(tabId, { file: 'js/element-picker.js' });
+    vAPI.tabs.injectScript(tabId, { file: 'js/scriptlets/element-picker.js' });
+    if ( typeof vAPI.tabs.select === 'function' ) {
+        vAPI.tabs.select(tabId);
+    }
 };
 
 /******************************************************************************/
@@ -306,6 +318,38 @@ var matchWhitelistDirective = function(url, hostname, directive) {
 
 /******************************************************************************/
 
+µBlock.toggleURLFilteringRule = function(details) {
+    var changed = this.sessionURLFiltering.setRule(
+        details.context,
+        details.url,
+        details.type,
+        details.action
+    );
+
+    if ( !changed ) {
+        return;
+    }
+
+    this.cosmeticFilteringEngine.removeFromSelectorCache(details.context, 'net');
+
+    if ( !details.persist ) {
+        return;
+    }
+
+    changed = this.permanentURLFiltering.setRule(
+        details.context,
+        details.url,
+        details.type,
+        details.action
+    );
+
+    if ( changed ) {
+        this.savePermanentFirewallRules();
+    }
+};
+
+/******************************************************************************/
+
 µBlock.isBlockResult = function(result) {
     return typeof result === 'string' && result.charAt(1) === 'b';
 };
@@ -326,7 +370,7 @@ var matchWhitelistDirective = function(url, hostname, directive) {
     // Take action if needed
     if ( details.name === 'no-cosmetic-filtering' ) {
         vAPI.tabs.injectScript(details.tabId, {
-            file: 'js/cosmetic-' + (details.state ? 'off' : 'on') + '.js',
+            file: 'js/scriptlets/cosmetic-' + (details.state ? 'off' : 'on') + '.js',
             allFrames: true
         });
         return;
@@ -338,13 +382,46 @@ var matchWhitelistDirective = function(url, hostname, directive) {
 
 /******************************************************************************/
 
-µBlock.getHiddenElementCount = function(tabId, callback) {
+µBlock.surveyCosmeticFilters = function(tabId, callback) {
     callback = callback || this.noopFunc;
     if ( vAPI.isBehindTheSceneTabId(tabId) ) {
         callback();
         return;
     }
-    vAPI.tabs.injectScript(tabId, { file: 'js/cosmetic-count.js' }, callback);
+    vAPI.tabs.injectScript(tabId, { file: 'js/scriptlets/cosmetic-survey.js' }, callback);
+};
+
+/******************************************************************************/
+
+µBlock.logCosmeticFilters = (function() {
+    var tabIdToTimerMap = {};
+
+    var injectNow = function(tabId) {
+        delete tabIdToTimerMap[tabId];
+        vAPI.tabs.injectScript(tabId, { file: 'js/scriptlets/cosmetic-logger.js' });
+    };
+
+    var injectAsync = function(tabId) {
+        if ( tabIdToTimerMap.hasOwnProperty(tabId) ) {
+            return;
+        }
+        tabIdToTimerMap[tabId] = vAPI.setTimeout(
+            injectNow.bind(null, tabId),
+            100
+        );
+    };
+
+    return injectAsync;
+})();
+
+/******************************************************************************/
+
+µBlock.scriptletGotoImageURL = function(details) {
+    if ( vAPI.isBehindTheSceneTabId(details.tabId) ) {
+        return;
+    }
+    this.scriptlets.gotoImageURL = details.url;
+    vAPI.tabs.injectScript(details.tabId, { file: 'js/scriptlets/goto-img.js' });
 };
 
 /******************************************************************************/
