@@ -42,6 +42,13 @@ var noopFunc = function(){};
 
 /******************************************************************************/
 
+// https://github.com/gorhill/uMatrix/issues/234
+// https://developer.chrome.com/extensions/privacy#property-network
+
+chrome.privacy.network.networkPredictionEnabled.set({ value: false });
+
+/******************************************************************************/
+
 vAPI.app = {
     name: manifest.name,
     version: manifest.version
@@ -73,6 +80,18 @@ vAPI.noTabId = '-1';
 
 /******************************************************************************/
 
+var toChromiumTabId = function(tabId) {
+    if ( typeof tabId === 'string' ) {
+        tabId = parseInt(tabId, 10);
+    }
+    if ( typeof tabId !== 'number' || isNaN(tabId) || tabId === -1 ) {
+        return 0;
+    }
+    return tabId;
+};
+
+/******************************************************************************/
+
 vAPI.tabs.registerListeners = function() {
     var onNavigationClient = this.onNavigation || noopFunc;
     var onPopupClient = this.onPopup || noopFunc;
@@ -204,17 +223,17 @@ vAPI.tabs.get = function(tabId, callback) {
         // Caller must be prepared to deal with nil tab value
         callback(tab);
     };
+
     if ( tabId !== null ) {
-        if ( typeof tabId === 'string' ) {
-            tabId = parseInt(tabId, 10);
-        }
-        if ( typeof tabId !== 'number' || isNaN(tabId) ) {
+        tabId = toChromiumTabId(tabId);
+        if ( tabId === 0 ) {
             onTabReady(null);
         } else {
             chrome.tabs.get(tabId, onTabReady);
         }
         return;
     }
+
     var onTabReceived = function(tabs) {
         // https://code.google.com/p/chromium/issues/detail?id=410868#c8
         if ( chrome.runtime.lastError ) {
@@ -274,7 +293,7 @@ vAPI.tabs.open = function(details) {
             }
 
             // update doesn't accept index, must use move
-            chrome.tabs.update(parseInt(details.tabId, 10), _details, function(tab) {
+            chrome.tabs.update(toChromiumTabId(details.tabId), _details, function(tab) {
                 // if the tab doesn't exist
                 if ( vAPI.lastError() ) {
                     chrome.tabs.create(_details, focusWindow);
@@ -322,6 +341,11 @@ vAPI.tabs.open = function(details) {
 // Replace the URL of a tab. Noop if the tab does not exist.
 
 vAPI.tabs.replace = function(tabId, url) {
+    tabId = toChromiumTabId(tabId);
+    if ( tabId === 0 ) {
+        return;
+    }
+
     var targetURL = url;
 
     // extension pages
@@ -329,17 +353,10 @@ vAPI.tabs.replace = function(tabId, url) {
         targetURL = vAPI.getURL(targetURL);
     }
 
-    if ( typeof tabId !== 'number' ) {
-        tabId = parseInt(tabId, 10);
-        if ( isNaN(tabId) ) {
-            return;
-        }
-    }
-
     chrome.tabs.update(tabId, { url: targetURL }, function() {
-        // this prevent console error
+        // https://code.google.com/p/chromium/issues/detail?id=410868#c8
         if ( chrome.runtime.lastError ) {
-            return;
+            /* noop */
         }
     });
 };
@@ -347,20 +364,58 @@ vAPI.tabs.replace = function(tabId, url) {
 /******************************************************************************/
 
 vAPI.tabs.remove = function(tabId) {
+    tabId = toChromiumTabId(tabId);
+    if ( tabId === 0 ) {
+        return;
+    }
+
     var onTabRemoved = function() {
-        if ( vAPI.lastError() ) {
+        // https://code.google.com/p/chromium/issues/detail?id=410868#c8
+        if ( chrome.runtime.lastError ) {
+            /* noop */
         }
     };
-    chrome.tabs.remove(parseInt(tabId, 10), onTabRemoved);
+
+    chrome.tabs.remove(tabId, onTabRemoved);
 };
 
 /******************************************************************************/
 
 vAPI.tabs.reload = function(tabId /*, flags*/) {
-    if ( typeof tabId === 'string' ) {
-        tabId = parseInt(tabId, 10);
+    tabId = toChromiumTabId(tabId);
+    if ( tabId === 0 ) {
+        return;
     }
-    chrome.tabs.reload(tabId);
+
+    var onReloaded = function() {
+        // https://code.google.com/p/chromium/issues/detail?id=410868#c8
+        if ( chrome.runtime.lastError ) {
+            /* noop */
+        }
+    };
+
+    chrome.tabs.reload(tabId, onReloaded);
+};
+
+/******************************************************************************/
+
+// Select a specific tab.
+
+vAPI.tabs.select = function(tabId) {
+    tabId = toChromiumTabId(tabId);
+    if ( tabId === 0 ) {
+        return;
+    }
+
+    chrome.tabs.update(tabId, { active: true }, function(tab) {
+        if ( chrome.runtime.lastError ) {
+            /* noop */
+        }
+        if ( !tab ) {
+            return;
+        }
+        chrome.windows.update(tab.windowId, { focused: true });
+    });
 };
 
 /******************************************************************************/
@@ -369,14 +424,14 @@ vAPI.tabs.injectScript = function(tabId, details, callback) {
     var onScriptExecuted = function() {
         // https://code.google.com/p/chromium/issues/detail?id=410868#c8
         if ( chrome.runtime.lastError ) {
+            /* noop */
         }
         if ( typeof callback === 'function' ) {
             callback();
         }
     };
     if ( tabId ) {
-        tabId = parseInt(tabId, 10);
-        chrome.tabs.executeScript(tabId, details, onScriptExecuted);
+        chrome.tabs.executeScript(toChromiumTabId(tabId), details, onScriptExecuted);
     } else {
         chrome.tabs.executeScript(details, onScriptExecuted);
     }
@@ -392,7 +447,11 @@ vAPI.tabs.injectScript = function(tabId, details, callback) {
 // anymore, so this ensures it does still exist.
 
 vAPI.setIcon = function(tabId, iconStatus, badge) {
-    tabId = parseInt(tabId, 10);
+    tabId = toChromiumTabId(tabId);
+    if ( tabId === 0 ) {
+        return;
+    }
+
     var onIconReady = function() {
         if ( vAPI.lastError() ) {
             return;
@@ -627,7 +686,7 @@ vAPI.net.registerListeners = function() {
         //},
         {
             'urls': this.onBeforeRequest.urls || ['<all_urls>'],
-            'types': this.onBeforeRequest.types || []
+            'types': this.onBeforeRequest.types || undefined
         },
         this.onBeforeRequest.extra
     );
@@ -696,12 +755,12 @@ vAPI.onLoadAllCompleted = function() {
         vAPI.tabs.injectScript(tabId, {
             file: 'js/vapi-client.js',
             allFrames: true,
-            runAt: 'document_start'
+            runAt: 'document_idle'
         }, function(){ });
         vAPI.tabs.injectScript(tabId, {
             file: 'js/contentscript-start.js',
             allFrames: true,
-            runAt: 'document_start'
+            runAt: 'document_idle'
         }, function(){ scriptEnd(tabId); });
     };
     var bindToTabs = function(tabs) {
@@ -716,8 +775,7 @@ vAPI.onLoadAllCompleted = function() {
         }
     };
 
-    chrome.tabs.query({ url: 'http://*/*' }, bindToTabs);
-    chrome.tabs.query({ url: 'https://*/*' }, bindToTabs);
+    chrome.tabs.query({ url: '<all_urls>' }, bindToTabs);
 };
 
 /******************************************************************************/
