@@ -30,10 +30,12 @@
 
 'use strict';
 
-self.vAPI = self.vAPI || {};
-
+var vAPI = self.vAPI = self.vAPI || {};
 var chrome = self.chrome;
-var vAPI = self.vAPI;
+
+/******************************************************************************/
+
+vAPI.setTimeout = vAPI.setTimeout || self.setTimeout.bind(self);
 
 /******************************************************************************/
 
